@@ -27,13 +27,9 @@
 
 #include <pthread.h>
 
-#define USEC_PER_SEC 1000000
-
 #define SYNCMQ_NAME "/syncmsg%d"
 #define TESTMQ_NAME "/testmsg%d"
 #define MSG_SIZE 8
-#define MSEC_PER_SEC 1000
-#define NSEC_PER_SEC 1000000000
 
 char *syncmsg = "Syncing";
 char *testmsg = "Testing";
@@ -68,7 +64,6 @@ struct params {
 	char recvsyncmsg[MSG_SIZE];
 	char recvtestmsg[MSG_SIZE];
 	struct params *neighbor;
-	char error[MAX_PATH * 2];
 };
 
 void *pmqthread(void *param)
@@ -87,7 +82,7 @@ void *pmqthread(void *param)
 	if (par->cpu != -1) {
 		CPU_ZERO(&mask);
 		CPU_SET(par->cpu, &mask);
-		if(sched_setaffinity(0, sizeof(mask), &mask) == -1)
+		if (sched_setaffinity(0, sizeof(mask), &mask) == -1)
 			fprintf(stderr,	"WARNING: Could not set CPU affinity "
 				"to CPU #%d\n", par->cpu);
 	} else
@@ -101,7 +96,7 @@ void *pmqthread(void *param)
 			/* Optionally force receiver timeout */
 			if (par->forcetimeout) {
 				struct timespec senddelay;
-				
+
 				senddelay.tv_sec = par->forcetimeout;
 				senddelay.tv_nsec = 0;
 				clock_nanosleep(CLOCK_MONOTONIC, 0, &senddelay, NULL);
@@ -114,7 +109,7 @@ void *pmqthread(void *param)
 				par->shutdown = 1;
 			}
 			par->samples++;
-			if(par->max_cycles && par->samples >= par->max_cycles)
+			if (par->max_cycles && par->samples >= par->max_cycles)
 				par->shutdown = 1;
 			if (mustgetcpu)
 				par->cpu = get_cpu();
@@ -122,17 +117,17 @@ void *pmqthread(void *param)
 			if (par->timeout) {
 				clock_gettime(CLOCK_REALTIME, &ts);
 				ts.tv_sec += par->timeout;
-	
+
 				if (mq_timedreceive(par->syncmq, par->recvsyncmsg, MSG_SIZE, NULL, &ts)
 				    != strlen(syncmsg)) {
 					fprintf(stderr, "could not receive sync message\n");
-					par->shutdown = 1;				
+					par->shutdown = 1;
 				}
-			} 
+			}
 			if (mq_receive(par->syncmq, par->recvsyncmsg, MSG_SIZE, NULL) !=
 			    strlen(syncmsg)) {
 				perror("could not receive sync message");
-				par->shutdown = 1;				
+				par->shutdown = 1;
 			}
 			if (!par->shutdown && strcmp(syncmsg, par->recvsyncmsg)) {
 				fprintf(stderr, "ERROR: Sync message mismatch detected\n");
@@ -160,8 +155,7 @@ void *pmqthread(void *param)
 						}
 					} else
 						break;
-				}
-				while (1);
+				} while (1);
 			} else {
 				if (mq_receive(par->testmq, par->recvtestmsg, MSG_SIZE, NULL) !=
 				    strlen(testmsg)) {
@@ -197,9 +191,7 @@ void *pmqthread(void *param)
 					write(tracing_enabled, "0", 1);
 					close(tracing_enabled);
 				} else
-					snprintf(par->error, sizeof(par->error),
-					    "Could not access %s\n",
-					    tracing_enabled_file);
+					fatal("Could not access %s\n", tracing_enabled_file);
 				par->shutdown = 1;
 				par->neighbor->shutdown = 1;
 			}
@@ -221,36 +213,37 @@ void *pmqthread(void *param)
 	return NULL;
 }
 
-
-static void display_help(void)
+static void display_help(int error)
 {
 	printf("pmqtest V %1.2f\n", VERSION);
-	puts("Usage: pmqtest <options>");
-	puts("Function: test POSIX message queue latency");
-	puts(
-	"Options:\n"
-	"-a [NUM] --affinity        run thread #N on processor #N, if possible\n"
-	"                           with NUM pin all threads to the processor NUM\n"
-	"-b USEC  --breaktrace=USEC send break trace command when latency > USEC\n"
-	"-d DIST  --distance=DIST   distance of thread intervals in us default=500\n"
-	"-f TO    --forcetimeout=TO force timeout of mq_timedreceive(), requires -T\n"
-	"-i INTV  --interval=INTV   base interval of thread in us default=1000\n"
-	"-l LOOPS --loops=LOOPS     number of loops: default=0(endless)\n"
-	"-D       --duration=TIME   specify a length for the test run.\n"
-	"                           Append 'm', 'h', or 'd' to specify minutes, hours or days.\n"
-	"-p PRIO  --prio=PRIO       priority\n"
-	"-S       --smp             SMP testing: options -a -t and same priority\n"
-        "                           of all threads\n"
-	"-t       --threads         one thread per available processor\n"
-	"-t [NUM] --threads=NUM     number of threads:\n"
-	"                           without NUM, threads = max_cpus\n"
-	"                           without -t default = 1\n"
-	"-T TO    --timeout=TO      use mq_timedreceive() instead of mq_receive()\n"
-	"                           with timeout TO in seconds\n");
-	exit(1);
+	printf("Usage:\n"
+	       "pmqtest <options>\n\n"
+	       "Function: test POSIX message queue latency\n\n"
+	       "Available options:\n\n"
+	       "-a [NUM] --affinity        run thread #N on processor #N, if possible\n"
+	       "                           with NUM pin all threads to the processor NUM\n"
+	       "-b USEC  --breaktrace=USEC send break trace command when latency > USEC\n"
+	       "-d DIST  --distance=DIST   distance of thread intervals in us default=500\n"
+	       "-D TIME  --duration=TIME   specify a length for the test run.\n"
+	       "                           Append 'm', 'h', or 'd' to specify\n"
+	       "                           minutes, hours or days.\n"
+	       "-f TO    --forcetimeout=TO force timeout of mq_timedreceive(), requires -T\n"
+	       "-h       --help            print this help message\n"
+	       "-i INTV  --interval=INTV   base interval of thread in us default=1000\n"
+	       "-l LOOPS --loops=LOOPS     number of loops: default=0(endless)\n"
+	       "-p PRIO  --prio=PRIO       priority\n"
+	       "-q       --quiet           print a summary only on exit\n"
+	       "-S       --smp             SMP testing: options -a -t and same priority\n"
+	       "                           of all threads\n"
+	       "-t       --threads         one thread per available processor\n"
+	       "-t [NUM] --threads=NUM     number of threads:\n"
+	       "                           without NUM, threads = max_cpus\n"
+	       "                           without -t default = 1\n"
+	       "-T TO    --timeout=TO      use mq_timedreceive() instead of mq_receive()\n"
+	       "                           with timeout TO in seconds\n");
+	exit(error);
 }
 
-
 static int setaffinity = AFFINITY_UNSPECIFIED;
 static int affinity;
 static int tracelimit;
@@ -264,8 +257,9 @@ static int smp;
 static int sameprio;
 static int timeout;
 static int forcetimeout;
+static int quiet;
 
-static void process_options (int argc, char *argv[])
+static void process_options(int argc, char *argv[])
 {
 	int error = 0;
 	int max_cpus = sysconf(_SC_NPROCESSORS_CONF);
@@ -274,21 +268,22 @@ static void process_options (int argc, char *argv[])
 		int option_index = 0;
 		/** Options for getopt */
 		static struct option long_options[] = {
-			{"affinity", optional_argument, NULL, 'a'},
-			{"breaktrace", required_argument, NULL, 'b'},
-			{"distance", required_argument, NULL, 'd'},
-			{"forcetimeout", required_argument, NULL, 'f'},
-			{"interval", required_argument, NULL, 'i'},
-			{"loops", required_argument, NULL, 'l'},
-			{"duration", required_argument, NULL, 'D'},
-			{"priority", required_argument, NULL, 'p'},
-			{"smp", no_argument, NULL, 'S'},
-			{"threads", optional_argument, NULL, 't'},
-			{"timeout", required_argument, NULL, 'T'},
-			{"help", no_argument, NULL, '?'},
+			{"affinity",		optional_argument,	NULL, 'a'},
+			{"breaktrace",		required_argument,	NULL, 'b'},
+			{"distance",		required_argument,	NULL, 'd'},
+			{"duration",		required_argument,	NULL, 'D'},
+			{"forcetimeout",	required_argument,	NULL, 'f'},
+			{"help",		no_argument,		NULL, 'h'},
+			{"interval",		required_argument,	NULL, 'i'},
+			{"loops",		required_argument,	NULL, 'l'},
+			{"priority",		required_argument,	NULL, 'p'},
+			{"quiet",		no_argument,		NULL, 'q'},
+			{"smp",			no_argument,		NULL, 'S'},
+			{"threads",		optional_argument,	NULL, 't'},
+			{"timeout",		required_argument,	NULL, 'T'},
 			{NULL, 0, NULL, 0}
 		};
-		int c = getopt_long (argc, argv, "a::b:d:f:i:l:D:p:St::T:",
+		int c = getopt_long (argc, argv, "a::b:d:D:f:i:l:p:qSt::T:",
 			long_options, &option_index);
 		if (c == -1)
 			break;
@@ -301,7 +296,7 @@ static void process_options (int argc, char *argv[])
 			if (optarg != NULL) {
 				affinity = atoi(optarg);
 				setaffinity = AFFINITY_SPECIFIED;
-			} else if (optind<argc && atoi(argv[optind])) {
+			} else if (optind < argc && atoi(argv[optind])) {
 				affinity = atoi(argv[optind]);
 				setaffinity = AFFINITY_SPECIFIED;
 			} else {
@@ -310,11 +305,14 @@ static void process_options (int argc, char *argv[])
 			break;
 		case 'b': tracelimit = atoi(optarg); break;
 		case 'd': distance = atoi(optarg); break;
+		case 'D': duration = parse_time_string(optarg); break;
 		case 'f': forcetimeout = atoi(optarg); break;
+		case '?':
+		case 'h': display_help(0); break;
 		case 'i': interval = atoi(optarg); break;
 		case 'l': max_cycles = atoi(optarg); break;
-		case 'D': duration = parse_time_string(optarg); break;
 		case 'p': priority = atoi(optarg); break;
+		case 'q': quiet = 1; break;
 		case 'S':
 			smp = 1;
 			num_threads = max_cpus;
@@ -327,13 +325,15 @@ static void process_options (int argc, char *argv[])
 			}
 			if (optarg != NULL)
 				num_threads = atoi(optarg);
-			else if (optind<argc && atoi(argv[optind]))
+			else if (optind < argc && atoi(argv[optind]))
 				num_threads = atoi(argv[optind]);
 			else
 				num_threads = max_cpus;
 			break;
 		case 'T': timeout = atoi(optarg); break;
-		case '?': error = 1; break;
+		default:
+			display_help(1);
+			break;
 		}
 	}
 
@@ -366,7 +366,7 @@ static void process_options (int argc, char *argv[])
 		sameprio = 1;
 
 	if (error)
-		display_help ();
+		display_help(error);
 }
 
 
@@ -377,6 +377,30 @@ static void sighand(int sig)
 	shutdown = 1;
 }
 
+static void print_stat(FILE *fp, struct params *receiver, struct params *sender,
+		       int verbose, int quiet)
+{
+	int i;
+
+	if (quiet)
+		return;
+
+	for (i = 0; i < num_threads; i++) {
+		printf("#%1d: ID%d, P%d, CPU%d, I%ld; #%1d: ID%d, P%d, CPU%d, TO %d, Cycles %d\n",
+			i*2, receiver[i].tid, receiver[i].priority, receiver[i].cpu,
+			receiver[i].delay.tv_nsec / 1000,
+			i*2+1, sender[i].tid, sender[i].priority, sender[i].cpu,
+			receiver[i].timeoutcount, sender[i].samples);
+	}
+	for (i = 0; i < num_threads; i++) {
+		printf("#%d -> #%d, Min %4d, Cur %4d, Avg %4d, Max %4d\n",
+			i*2+1, i*2,
+			receiver[i].mindiff, (int) receiver[i].diff.tv_usec,
+			(int) ((receiver[i].sumdiff / receiver[i].samples) + 0.5),
+			receiver[i].maxdiff);
+	}
+}
+
 int main(int argc, char *argv[])
 {
 	int i;
@@ -386,8 +410,6 @@ int main(int argc, char *argv[])
 	sigset_t sigset;
 	int oldsamples = INT_MAX;
 	int oldtimeoutcount = INT_MAX;
-	int first = 1;
-	int errorlines = 0;
 	struct timespec maindelay;
 	int oflag = O_CREAT|O_RDWR;
 	struct mq_attr mqstat;
@@ -490,40 +512,11 @@ int main(int argc, char *argv[])
 
 		if (minsamples > 1 && (shutdown || newsamples > oldsamples ||
 			newtimeoutcount > oldtimeoutcount)) {
-
-			if (!first)
-				printf("\033[%dA", num_threads*2 + errorlines);
-			first = 0;
-
-			for (i = 0; i < num_threads; i++) {
-				printf("#%1d: ID%d, P%d, CPU%d, I%ld; #%1d: ID%d, P%d, CPU%d, TO %d, Cycles %d   \n",
-				    i*2, receiver[i].tid, receiver[i].priority, receiver[i].cpu,
-				    receiver[i].delay.tv_nsec / 1000,
-				    i*2+1, sender[i].tid, sender[i].priority, sender[i].cpu,
-				    receiver[i].timeoutcount, sender[i].samples);
-			}
-			for (i = 0; i < num_threads; i++) {
-				printf("#%d -> #%d, Min %4d, Cur %4d, Avg %4d, Max %4d\n",
-					i*2+1, i*2,
-					receiver[i].mindiff, (int) receiver[i].diff.tv_usec,
-					(int) ((receiver[i].sumdiff / receiver[i].samples) + 0.5),
-					receiver[i].maxdiff);
-				if (receiver[i].error[0] != '\0') {
-					printf("%s", receiver[i].error);
-					errorlines++;
-					receiver[i].error[0] = '\0';
-				}
-				if (sender[i].error[0] != '\0') {
-					printf("%s", sender[i].error);
-					errorlines++;
-					receiver[i].error[0] = '\0';
-				}
-			}
-		} else {
-			if (minsamples < 1)
-				printf("Collecting ...\n\033[1A");
+			print_stat(stdout, receiver, sender, 0, quiet);
+			if (!quiet)
+				printf("\033[%dA", num_threads*2);
 		}
-		
+
 		fflush(NULL);
 
 		oldsamples = 0;
@@ -540,6 +533,11 @@ int main(int argc, char *argv[])
 
 	} while (!shutdown);
 
+	if (!quiet)
+		printf("\033[%dB", num_threads*2 + 2);
+	else
+		print_stat(stdout, receiver, sender, 0, 0);
+
 	for (i = 0; i < num_threads; i++) {
 		receiver[i].shutdown = 1;
 		sender[i].shutdown = 1;
@@ -564,7 +562,7 @@ int main(int argc, char *argv[])
 		mq_unlink(mqname);
 	}
 
-	nomem:
+nomem:
 
 	return 0;
 }
