@@ -1,22 +1,12 @@
+// SPDX-License-Identifier: GPL-2.0-only
+
 /*
  * Copyright (C) 2016 Red Hat Inc, Steven Rostedt <srostedt@redhat.com>
- *
- * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License (not later!)
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not,  see <http://www.gnu.org/licenses>
- *
- * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
- *
+ * Copyright (C) 2019 John Kacur <jkacur@redhat.com>
+ * Copyright (C) 2019 Clark Williams <williams@redhat.com>
+ */
+
+/*
  * deadline_test.c
  *
  * This program is used to test the deadline scheduler (SCHED_DEADLINE tasks).
@@ -51,74 +41,37 @@
 #include <linux/unistd.h>
 #include <linux/magic.h>
 
+#include <rt-utils.h>
+#include <rt-sched.h>
+
 /**
  * usage - show the usage of the program and exit.
- * @argv: The program passed in args
+ * @error: Exit error code to be used
  *
- * This is defined here to show people looking at this code how
- * to use this program as well. 
+ * This is defined here to show peoplde looking at this code how
+ * to use this program as well.
  */
-static void usage(char **argv)
+static void usage(int error)
 {
-	char *arg = argv[0];
-	char *p = arg+strlen(arg);
-
-	while (p >= arg && *p != '/')
-		p--;
-	p++;
-
-	printf("usage: %s [options]\n"
-	       " -h - Show this help menu\n"
-	       " -b - Bind on the last cpu. (shortcut for -c <lastcpu>)\n"
-	       " -r prio - Add an RT task with given prio to stress system\n"
-	       " -c cpulist - Comma/hyphen separated list of CPUs to run deadline tasks on\n"
-	       " -i interval - The shortest deadline for the tasks\n"
-	       " -p percent - The percent of bandwidth to use (1-90%%)\n"
-	       " -P percent - The percent of runtime for execution completion\n"
-	       "              (Default 100%%)\n"
-	       " -t threads - The number of threads to run as deadline (default 1)\n"
-	       " -s step(us) - The amount to increase the deadline for each task (default 500us)\n"
-	       "\n", p);
-	exit(-1);
+	printf("deadline_test V %1.2f\n", VERSION);
+	printf("Usage:\n"
+	       "deadline_test <options>\n"
+	       "-b                         Bind on the last cpu. (shortcut for -c <lastcpu>)\n"
+	       "-c CPUSET                  Comma/hyphen separated list of CPUs to run deadline\n"
+	       "                           tasks on\n"
+	       "-h                         Show this help menu\n"
+	       "-i INTV                    The shortest deadline for the tasks\n"
+	       "-p PERCENT                 The percent of bandwidth to use (1-90%%)\n"
+	       "-P PERCENT                 The percent of runtime for execution completion\n"
+	       "                           (default 100%%)\n"
+	       "-r PRIO                    Add an RT task with given prio to stress system\n"
+	       "-s STEP                    The amount to increase the deadline for each task in us\n"
+	       "                           (default 500us)\n"
+	       "-t NUM                     The number of threads to run as deadline (default 1)\n"
+	       );
+	exit(error);
 }
 
-/*
- * sched_setattr() and sched_getattr() are new system calls. We need to
- * hardcode it here.
- */
-#if defined(__i386__)
-
-#ifndef __NR_sched_setattr
-#define __NR_sched_setattr		351
-#endif
-#ifndef __NR_sched_getattr
-#define __NR_sched_getattr		352
-#endif
-
-#elif defined(__x86_64__)
-
-#ifndef __NR_sched_setattr
-#define __NR_sched_setattr		314
-#endif
-#ifndef __NR_sched_getattr
-#define __NR_sched_getattr		315
-#endif
-
-#endif /* i386 or x86_64 */
-
-#if !defined(__NR_sched_setattr)
-# error "Your arch does not support sched_setattr()"
-#endif
-
-#if !defined(__NR_sched_getattr)
-# error "Your arch does not support sched_getattr()"
-#endif
-
-/* If not included in the headers, define sched deadline policy numbe */
-#ifndef SCHED_DEADLINE
-#define SCHED_DEADLINE		6
-#endif
-
 #define _STR(x) #x
 #define STR(x) _STR(x)
 
@@ -138,46 +91,10 @@ static void usage(char **argv)
 #define CPUSET_ALL	"my_cpuset_all"
 #define CPUSET_LOCAL	"my_cpuset"
 
-/* Define the system call interfaces */
-#define gettid() syscall(__NR_gettid)
-#define sched_setattr(pid, attr, flags) syscall(__NR_sched_setattr, pid, attr, flags)
-#define sched_getattr(pid, attr, size, flags) syscall(__NR_sched_getattr, pid, attr, size, flags)
-
 typedef unsigned long long u64;
 typedef unsigned int u32;
 typedef int s32;
 
-/**
- * struct sched_attr - get/set attr system call descriptor.
- *
- * This is the descriptor defined for setting SCHED_DEADLINE tasks.
- * It will someday be in a header file.
- *
- * The fields specific for deadline:
- *
- *  @sched_policy: 6 is for deadline
- *  @sched_runtime: The runtime in nanoseconds
- *  @sched_deadline: The deadline in nanoseconds.
- *  @sched_period: The period, if different than deadline (not used here)
- */
-struct sched_attr {
-	u32 size;
-
-	u32 sched_policy;
-	u64 sched_flags;
-
-	/* SCHED_NORMAL, SCHED_BATCH */
-	s32 sched_nice;
-
-	/* SCHED_FIFO, SCHED_RR */
-	u32 sched_priority;
-
-	/* SCHED_DEADLINE */
-	u64 sched_runtime;
-	u64 sched_deadline;
-	u64 sched_period;
-};
-
 /**
  * struct sched_data - the descriptor for the threads.
  *
@@ -268,7 +185,7 @@ static int find_mount(const char *mount, char *debugfs)
 	char type[100];
 	FILE *fp;
 
-	if ((fp = fopen("/proc/mounts","r")) == NULL)
+	if ((fp = fopen("/proc/mounts", "r")) == NULL)
 		return 0;
 
 	while (fscanf(fp, "%*s %"
@@ -301,7 +218,7 @@ static const char *find_debugfs(void)
 
 	if (!find_mount("debugfs", debugfs))
 		return "";
-	
+
 	debugfs_found = 1;
 
 	return debugfs;
@@ -435,7 +352,7 @@ static void setup_ftrace_marker(void)
 {
 	struct stat st;
 	const char *debugfs = find_debugfs();
-	char files[strlen(debugfs) + 14];
+	char files[strlen(debugfs) + strlen("/tracing/trace_marker") + 1];
 	int ret;
 
 	if (strlen(debugfs) == 0)
@@ -788,8 +705,8 @@ static const char *make_cpuset(const char *name, const char *cpus,
 		fd = open_cpuset(path, "tasks");
 
 		snprintf(path, MAXPATH - 1, "%s/tasks", CPUSET_PATH);
-		if ((fp = fopen(path,"r")) == NULL) {
-			close (fd);
+		if ((fp = fopen(path, "r")) == NULL) {
+			close(fd);
 			return "opening cpuset tasks";
 		}
 
@@ -851,7 +768,7 @@ static void destroy_cpuset(const char *name, int print)
 	 * in this cpuset, that must be moved before destroying the cpuset.
 	 */
 	strncat(path, "/tasks", MAXPATH - 1);
-	if ((fp = fopen(path,"r")) == NULL) {
+	if ((fp = fopen(path, "r")) == NULL) {
 		fprintf(stderr, "Failed opening %s\n", path);
 		perror("fopen");
 		return;
@@ -1335,7 +1252,7 @@ void *run_deadline(void *data)
 	attr.sched_runtime = sched_data->runtime_us * 1000;
 	attr.sched_deadline = sched_data->deadline_us * 1000;
 
-	printf("thread[%ld] runtime=%lldus deadline=%lldus loops=%lld\n",
+	printf("thread[%d] runtime=%lldus deadline=%lldus loops=%lld\n",
 	       gettid(), sched_data->runtime_us,
 	       sched_data->deadline_us, sched_data->loops_per_period);
 
@@ -1359,7 +1276,7 @@ void *run_deadline(void *data)
 
 	sched_yield();
 	period = get_time_us();
-	
+
 	while (!done) {
 		period = do_runtime(tid, sched_data, period);
 		sched_yield();
@@ -1418,9 +1335,8 @@ void *run_rt_spin(void *data)
 	if (fail)
 		return NULL;
 
-	while (!done) {
+	while (!done)
 		get_time_us();
-	}
 
 	return NULL;
 }
@@ -1772,7 +1688,7 @@ static u64 calculate_loops_per_ms(u64 *overhead)
 	return loops;
 }
 
-int main (int argc, char **argv)
+int main(int argc, char **argv)
 {
 	struct sched_data *sched_data;
 	struct sched_data *sd;
@@ -1833,8 +1749,10 @@ int main (int argc, char **argv)
 			rt_task = atoi(optarg);
 			break;
 		case 'h':
+			usage(0);
+			break;
 		default:
-			usage(argv);
+			usage(1);
 		}
 	}
 
@@ -1865,7 +1783,7 @@ int main (int argc, char **argv)
 
 	/* -b has us bind to the last CPU. */
 	if (!all_cpus && !setcpu) {
-		setcpu_buf = malloc(10);
+		setcpu_buf = malloc(12);
 		if (!setcpu_buf) {
 			perror("malloc");
 			exit(-1);
@@ -1900,9 +1818,8 @@ int main (int argc, char **argv)
 		exit(-1);
 	}
 
-	if (mlockall(MCL_CURRENT|MCL_FUTURE) == -1) {
+	if (mlockall(MCL_CURRENT|MCL_FUTURE) == -1)
 		perror("mlockall");
-	}
 
 	/*
 	 * Run at prio 99 bound to the last CPU, and try to calculate
@@ -2001,9 +1918,8 @@ int main (int argc, char **argv)
 
 	pthread_barrier_wait(&barrier);
 
-	if (fail) {
+	if (fail)
 		exit(-1);
-	}
 
 	if (!all_cpus) {
 		int *pids;
@@ -2092,6 +2008,12 @@ int main (int argc, char **argv)
 		printf("\n");
 	}
 
-	free(setcpu_buf);
+	if (setcpu_buf)
+		free(setcpu_buf);
+	free(thread);
+	free(sched_data);
+
+	CPU_FREE(cpusetp);
+
 	return 0;
 }
